"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const safe_type_1 = require("@paulpopat/safe-type");
function GenerateUrlType(schema, metadata, headers, api) {
    return async (parameters) => {
        try {
            let url = schema.url;
            for (const key in parameters) {
                if (!parameters.hasOwnProperty(key)) {
                    continue;
                }
                const input = parameters[key];
                if ((0, safe_type_1.IsString)(input)) {
                    const p = encodeURIComponent(input);
                    const k = encodeURIComponent(key);
                    if (url.indexOf(":" + key) > -1) {
                        url = url.replace(":" + key, p);
                        continue;
                    }
                    if (url.indexOf("?") > -1) {
                        url += "&" + k + "=" + p;
                        continue;
                    }
                    url += "?" + k + "=" + p;
                }
                else if ((0, safe_type_1.IsArray)(safe_type_1.IsString)(input)) {
                    for (const i of input) {
                        const p = encodeURIComponent(i);
                        const k = encodeURIComponent(key);
                        if (url.indexOf(":" + key) > -1) {
                            url = url.replace(":" + key, p);
                            continue;
                        }
                        if (url.indexOf("?") > -1) {
                            url += "&" + k + "=" + p;
                            continue;
                        }
                        url += "?" + k + "=" + p;
                    }
                }
            }
            if (url.indexOf(":") > -1) {
                throw new Error(`Parameter has not been filled in ${url}`);
            }
            const { data: result } = await api.request({
                method: schema.method,
                url: url,
                baseURL: metadata.base,
                headers: headers(),
            });
            (0, safe_type_1.Assert)(schema.returns, result);
            return result;
        }
        catch (err) {
            if (metadata.on_error)
                throw metadata.on_error(err, schema.url);
        }
    };
}
function GenerateBodyType(schema, metadata, headers, api) {
    return async (parameters, body) => {
        try {
            let url = schema.url;
            for (const key in parameters) {
                if (!parameters.hasOwnProperty(key)) {
                    continue;
                }
                const input = parameters[key];
                if (!(0, safe_type_1.IsString)(input)) {
                    throw new Error("Cannot have array parameters or null parameters of the url");
                }
                const p = encodeURIComponent(input);
                if (url.indexOf(":" + key) > -1) {
                    url = url.replace(":" + key, p);
                    continue;
                }
                throw new Error(`Parameter ${key} does not exist in ${schema.url}`);
            }
            if (url.indexOf(":") > -1) {
                throw new Error(`Parameter has not been filled in ${url}`);
            }
            const { data: result } = await api.request({
                method: schema.method,
                url: url,
                data: body,
                baseURL: metadata.base,
                headers: headers(),
            });
            (0, safe_type_1.Assert)(schema.returns, result);
            return result;
        }
        catch (err) {
            if (metadata.on_error)
                throw metadata.on_error(err, schema.url);
        }
    };
}
function GeneratePlainUrlType(schema, metadata, headers, api) {
    return async () => {
        try {
            const { data: result } = await api.request({
                method: schema.method,
                url: schema.url,
                baseURL: metadata.base,
                headers: headers(),
            });
            (0, safe_type_1.Assert)(schema.returns, result);
            return result;
        }
        catch (err) {
            if (metadata.on_error)
                throw metadata.on_error(err, schema.url);
        }
    };
}
function GeneratePlainBodyType(schema, metadata, headers, api) {
    return async (body) => {
        try {
            const { data: result } = await api.request({
                method: schema.method,
                url: schema.url,
                baseURL: metadata.base,
                data: body,
                headers: headers(),
            });
            (0, safe_type_1.Assert)(schema.returns, result);
            return result;
        }
        catch (err) {
            if (metadata.on_error)
                throw metadata.on_error(err, schema.url);
        }
    };
}
function IsBody(schema) {
    return schema.hasOwnProperty("body") && schema.hasOwnProperty("parameters");
}
function IsUrl(schema) {
    return !schema.hasOwnProperty("body") && schema.hasOwnProperty("parameters");
}
function IsPlainUrl(schema) {
    return (!schema.hasOwnProperty("parameters") &&
        !schema.hasOwnProperty("body") &&
        schema.hasOwnProperty("url"));
}
function IsPlainBody(schema) {
    return !schema.hasOwnProperty("parameters") && schema.hasOwnProperty("body");
}
function IsApis(schema) {
    return (!IsBody(schema) &&
        !IsUrl(schema) &&
        !IsPlainUrl(schema) &&
        !IsPlainBody(schema));
}
function GenerateInterface(schema, metadata, default_headers, instance) {
    const result = {};
    let headers = {};
    for (const key in schema) {
        if (!schema.hasOwnProperty(key)) {
            continue;
        }
        const api = schema[key];
        if (IsPlainBody(api)) {
            result[key] = GeneratePlainBodyType(api, metadata, () => ({ ...default_headers, ...headers }), instance);
        }
        else if (IsPlainUrl(api)) {
            result[key] = GeneratePlainUrlType(api, metadata, () => ({ ...default_headers, ...headers }), instance);
        }
        else if (IsBody(api)) {
            result[key] = GenerateBodyType(api, metadata, () => ({ ...default_headers, ...headers }), instance);
        }
        else if (IsUrl(api)) {
            result[key] = GenerateUrlType(api, metadata, () => ({ ...default_headers, ...headers }), instance);
        }
        else if (IsApis(api)) {
            result[key] = GenerateInterface(api, metadata, default_headers, instance);
        }
    }
    Object.defineProperty(result, "headers", {
        get: function () {
            return { ...headers };
        },
        set: function (v) {
            headers = v;
        },
    });
    return result;
}
function default_1(schema, metadata) {
    const instance = axios_1.default.create({ baseURL: metadata.base });
    if (metadata.on_request)
        instance.interceptors.request.use(metadata.on_request);
    if (metadata.on_response)
        instance.interceptors.response.use(metadata.on_response);
    return GenerateInterface(schema, metadata, metadata.headers || {}, instance);
}
exports.default = default_1;
